#[path = "integration/anchor_dynamic_ref_test_schema.rs"]
mod anchor_dynamic_ref_test_schema;
#[path = "integration/array_const_enum_test_schema.rs"]
mod array_const_enum_test_schema;
#[path = "integration/cargo_schema.rs"]
mod cargo_schema;
#[path = "integration/contains_test_schema.rs"]
mod contains_test_schema;
#[path = "integration/dependencies_strict_mode_test_schema.rs"]
mod dependencies_strict_mode_test_schema;
#[path = "integration/dependencies_test_schema.rs"]
mod dependencies_test_schema;
#[path = "integration/dependent_required_test_schema.rs"]
mod dependent_required_test_schema;
#[path = "integration/dependent_schemas_test_schema.rs"]
mod dependent_schemas_test_schema;
#[path = "integration/format_annotation_test_schema.rs"]
mod format_annotation_test_schema;
#[path = "integration/min_max_contains_test_schema.rs"]
mod min_max_contains_test_schema;
#[path = "integration/non_schema.rs"]
mod non_schema;
#[path = "integration/other_schema.rs"]
mod other_schema;
#[path = "integration/prefix_items_test_schema.rs"]
mod prefix_items_test_schema;
#[path = "integration/recursive_anchor_ref_test_schema.rs"]
mod recursive_anchor_ref_test_schema;
#[path = "integration/string_format_test_schema.rs"]
mod string_format_test_schema;
#[path = "integration/table_const_enum_test_schema.rs"]
mod table_const_enum_test_schema;
#[path = "integration/tombi_schema.rs"]
mod tombi_schema;
#[path = "integration/tuple_items_test_schema.rs"]
mod tuple_items_test_schema;
#[path = "integration/untagged_union_schema.rs"]
mod untagged_union_schema;
